@@ -29,10 +29,15 @@ def name_folder_safe(foldername):
         foldername = foldername.replace("__", "_")
     return foldername
 
-def set_node_default(OverlayGraph, nbunch, **kwargs):
-    """Sets all nodes in nbunch to value if key not already set"""
+def set_node_default(OverlayGraph, nbunch = None, **kwargs):
+    """Sets all nodes in nbunch to value if key not already set
+    Note: this won't apply to future nodes added
+    """
     graph = unwrap_graph(OverlayGraph)
-    nbunch = unwrap_nodes(nbunch)
+    if nbunch is None:
+        nbunch = graph.nodes()
+    else:
+        nbunch = unwrap_nodes(nbunch)
     for node in nbunch:
         for key, val in kwargs.items():
             if key not in graph.node[node]:
@@ -42,7 +47,7 @@ def set_node_default(OverlayGraph, nbunch, **kwargs):
 #TODO: also add ability to copy multiple attributes
 
 #TODO: rename to copy_node_attr_from
-def copy_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, nbunch = None, type = None):
+def copy_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, nbunch = None, type = None, default = None):
     #TODO: add dest format, eg to convert to int
     if not dst_attr:
         dst_attr = src_attr
@@ -54,7 +59,7 @@ def copy_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, nbunch =
 
     for n in nbunch:
         try:
-            val = graph_src.node[n][src_attr]
+            val = graph_src.node[n].get(src_attr, default)
         except KeyError:
             #TODO: check if because node doesn't exist in dest, or because attribute doesn't exist in graph_src
             log.debug("Unable to copy node attribute %s for %s in %s" % (src_attr, n, overlay_src))
@@ -68,7 +73,7 @@ def copy_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, nbunch =
             if n in graph_dst:
                 graph_dst.node[n][dst_attr] = val
 
-def copy_edge_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, type = None):
+def copy_edge_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, type = None, default = None):
     graph_src = unwrap_graph(overlay_src)
     graph_dst = unwrap_graph(overlay_dst)
     if not dst_attr:
@@ -76,7 +81,7 @@ def copy_edge_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, typ
 
     for src, dst in graph_src.edges():
         try:
-            val = graph_src[src][dst][src_attr]
+            val = graph_src[src][dst].get(src_attr, default)
         except KeyError:
             #TODO: check if because edge doesn't exist in dest, or because attribute doesn't exist in graph_src
             log.debug("Unable to copy edge attribute %s for (%s, %s) in %s" % (src_attr, src, dst, overlay_src))
@@ -91,74 +96,6 @@ def copy_edge_attr_from(overlay_src, overlay_dst, src_attr, dst_attr = None, typ
             else:
                 log.debug("Unable to copy edge attribute %s for (%s, %s) not in target graph %s" % (src_attr, src, dst, overlay_dst))
 
-
-def stringify_netaddr(graph):
-    import netaddr
-# converts netaddr from iterables to strings so can use with json
-    replace_as_string = set([netaddr.ip.IPAddress, netaddr.ip.IPNetwork])
-#TODO: see if should handle dict specially, eg expand to __ ?
-
-    for key, val in graph.graph.items():
-        if type(val) in replace_as_string:
-            graph.graph[key] = str(val)
-
-    for node, data in graph.nodes(data=True):
-        for key, val in data.items():
-            if type(val) in replace_as_string:
-                graph.node[node][key] = str(val)
-
-    for src, dst, data in graph.edges(data=True):
-        for key, val in data.items():
-            if type(val) in replace_as_string:
-                graph[src][dst][key] = str(val)
-
-    return graph
-
-def save(OverlayGraph):
-    import netaddr
-    graph = OverlayGraph._graph.copy() # copy as want to annotate
-
-# and put in basic attributes
-    for node in OverlayGraph:
-        data = {}
-        data['label'] = node.label
-        #TODO: make these come from G_phy instead
-        #graph.node[node.node_id]['label'] = node.overlay.input.label
-        #graph.node[node.node_id]['device_type'] = node.overlay.input.device_type
-        graphics_node = node.anm['graphics'].node(node)
-        graph.node[node.node_id]['device_type'] = graphics_node.device_type
-        graph.node[node.node_id]['x'] = graphics_node.x
-        graph.node[node.node_id]['y'] = graphics_node.y
-
-        graph.node[node.node_id].update(data)
-#TODO: tidy this up
-
-    replace_as_string = set([type(None), netaddr.ip.IPAddress, netaddr.ip.IPNetwork, dict, defaultdict])
-#TODO: see if should handle dict specially, eg expand to __ ?
-
-    for key, val in graph.graph.items():
-        if type(val) in replace_as_string:
-            graph.graph[key] = str(val)
-
-    for node, data in graph.nodes(data=True):
-        for key, val in data.items():
-            if type(val) in replace_as_string:
-                graph.node[node][key] = str(val)
-
-    for src, dst, data in graph.edges(data=True):
-        for key, val in data.items():
-            if type(val) in replace_as_string:
-                graph[src][dst][key] = str(val)
-
-    mapping = dict( (n.node_id, str(n)) for n in OverlayGraph)
-    nx.relabel_nodes( graph, mapping, copy=False)
-#TODO: See why getting networkx.exception.NetworkXError: GraphML writer does not support <type 'NoneType'> as data values.
-#TODO: process writer to allow writing of IPnetwork class values
-    #filename = "%s.graphml" % OverlayGraph.name
-    #nx.write_graphml(graph, filename)
-
-# probably want to create a graph from input with switches expanded to direct connections
-
 #TODO: make edges own module
 def wrap_edges(OverlayGraph, edges):
     """ wraps edge ids into edge overlay """
@@ -276,6 +213,25 @@ def explode_nodes(OverlayGraph, nodes, retain = []):
 def label(OverlayGraph, nodes):
     return list(OverlayGraph._anm.node_label(node) for node in nodes)
 
+def connected_subgraphs(OverlayGraph, nodes):
+    nodes = list(unwrap_nodes(nodes))
+    graph = unwrap_graph(OverlayGraph)
+    subgraph = graph.subgraph(nodes)
+    if not len(subgraph.edges()):
+        #print "Nothing to aggregate for %s: no edges in subgraph"
+        pass
+    total_added_edges = []
+    if graph.is_directed():
+        component_nodes_list = nx.strongly_connected_components(subgraph)
+    else:
+        component_nodes_list = nx.connected_components(subgraph)
+
+    wrapped = []
+    for component in component_nodes_list:
+        wrapped.append(list(wrap_nodes(OverlayGraph, component)))
+
+    return wrapped
+
 def aggregate_nodes(OverlayGraph, nodes, retain = []):
     """Combines connected into a single node"""
     try:
@@ -351,8 +307,9 @@ def most_frequent(iterable):
         log.warning("Unable to calculate most_frequent, %s" % e)
         return None
 
-def neigh_most_frequent(OverlayGraph, node, attribute, attribute_graph = None):
+def neigh_most_frequent(OverlayGraph, node, attribute, attribute_graph = None, allow_none = False):
     """Used to explicitly force most frequent - useful if integers such as ASN which would otherwise return mean"""
+    #TODO: rename to median?
     graph = unwrap_graph(OverlayGraph)
     if attribute_graph:
         attribute_graph = unwrap_graph(attribute_graph)
@@ -360,6 +317,9 @@ def neigh_most_frequent(OverlayGraph, node, attribute, attribute_graph = None):
         attribute_graph = graph # use input graph
     node = unwrap_nodes(node)
     values = [attribute_graph.node[n].get(attribute) for n in graph.neighbors(node)]
+    values = sorted(values)
+    if not allow_none:
+        values = [v for v in values if v is not None]
     return most_frequent(values)
 
 
